#define F_CPU 12000000
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

// w肳ꂽԂ҂
void delay_ms(uint16_t w){
	while (w-->0) _delay_ms(1);
}

/////////////////////////// LCD Functions /////////////////////////////
#define LCD_PORT PORTD
#define LCD_SIG_RS 0x10
#define LCD_SIG_E  0x20
#define LCD_CMD_CLEAR 0x01

volatile void lcd_wait(void)
{
	_delay_us(40);
}

// 8rbgo
void lcd_out(unsigned char d)
{
	LCD_PORT = d | LCD_SIG_E; // output and enable E
	lcd_wait();
	LCD_PORT &=  ~LCD_SIG_E; // output and disable E
	lcd_wait();
}

// LCD
void lcd_init(void)
{
	// 3 00000011o
	lcd_out(0x03);
	_delay_ms(5);
	lcd_out(0x03);
	_delay_ms(2);
	lcd_out(0x03);
	lcd_wait();

	// t@NVZbg
	lcd_out(0x02); // 4rbg[h
	lcd_out(0x2);
	lcd_out(0x8);
	lcd_wait();

	// 00001100 \I/J[\OFF/uNOFF
	lcd_out(0x0);
	lcd_out(0xc);

	// 00000110 Gg[[hZbg
	lcd_out(0x0);
	lcd_out(0x6);
}

// f[^o
void lcd_write(unsigned char d)
{
	lcd_out(((d >> 4)&0x0f) | LCD_SIG_RS);  // 4rbg
	lcd_out((d&0x0f) | LCD_SIG_RS);       // 4rbg
	lcd_wait();
}

// \NA
void lcd_clear(void)
{
	LCD_PORT = 0;
	lcd_out(0);
	lcd_out(LCD_CMD_CLEAR);
	_delay_ms(2);
}


void lcd_setpos(int x,int y)
{
	y &= 0x1; // 2ŝ
	LCD_PORT = 0;
	lcd_out(0x08 | (y << 2) );
	lcd_out(x);
	_delay_ms(1);
}

static int lcd_line = 0;
static int lcd_clearflag = 0;

// charM
void lcd_putchar(char c) {
	if (c=='\n') {
		lcd_line++;
		lcd_clearflag = 1;
	} else if (c=='\f') {
		lcd_clear();
		lcd_line = 0;
	} else {
		if (lcd_clearflag) {
			lcd_setpos(0,lcd_line);
			lcd_clearflag = 17;
			while (--lcd_clearflag) lcd_write(' ');
			lcd_setpos(0,lcd_line);
		}
		lcd_write((unsigned char)c);
	}
}

// o
void lcd_puts(char *s) {
	while (*s) lcd_putchar(*s++);
}


// lM
void lcd_puti(long d)
{
	char s[8];
	int l=0,sgf=0;
	s[sizeof(s)-1]='\0';
	if (d<0) {
		d=-d;
		sgf=1;
	}
	do{
		s[sizeof(s)-2-l]=d%10+'0';
		l++;
	} while(d/=10);
	if (sgf) {
		s[sizeof(s)-2-l]='-';
		l++;
	}
	lcd_puts(s+(sizeof(s)-1-l));
}
///////////////////////////////////////////////////////////////////////

void pwm_init()
{
	TCNT0 = 0x00; //JE^
	TCCR0A  = (1<<COM0A1);
	TCCR0A |= (1<<WGM00); //ʑPWM
	TCCR0B  = (1<<CS01); //Ou1/8
}

int main()
{
	DDRD = 0xff;  // LCD

	delay_ms(10); // ҂
	lcd_init();
	lcd_clear();

	// obNCgPWMŐ
	pwm_init();
	OCR0A=0x00;

	// Xɖ邭
	uint8_t i;
	for (i=0;i<255;i++) {
		delay_ms(5);
		OCR0A=i;
	}

	delay_ms(50);

	lcd_puts("Stand by ready!\n");

	delay_ms(500);

	lcd_puts("Set up.");

	for(;;);
}
