#define F_CPU 12000000

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "usbdrv.h"

static unsigned char usbData[8];


// USB Report descriptor
PROGMEM char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] = {
	0x06, 0xA0, 0xFF,	// USAGE_PAGE (FFA0H Vendor defined)
	0x09, 0x01,			// USAGE (Vendor Usage 1)
	0xA1, 0x01,			// COLLECTION (Application)
		0x09, 0x02,			// USAGE (vendor defined)
		0xA1, 0x00,			// COLLECTION (Physical)
			0x06, 0xA1, 0xFF,	// USAGE_PAGE (vendor defined)
		// Input report
			0x09, 0x03,			// USAGE (vendor defined)
			0x09, 0x04,			// USAGE (vendor defined)
			0x15, 0x80,			// LOGICAL_MINIMUM (-128)
			0x25, 0x7F,			// LOGICAL_MAXIMUM (127)
			0x35, 0x00,			// PHYSICAL_MINIMUM (0)
			0x45, 0xFF,			// PHYSICAL_MAXIMUM (255)
			0x75, 0x08,			// REPORT_SIZE (8bit)
			0x95, 0x08,			// REPORT_COUNT (8 fields)
			0x81, 0x02,			// INPUT (Data, Variable, Absolute)
		// Output report
			0x09, 0x05,			// USAGE (vendor defined)
			0x09, 0x06,			// USAGE (vendor defined)
			0x15, 0x80,			// LOGICAL_MINIMUM (-128)
			0x25, 0x7F,			// LOGICAL_MAXIMUM (127)
			0x35, 0x00,			// PHYSICAL_MINIMUM (0)
			0x45, 0xFF,			// PHYSICAL_MAXIMUM (255)
			0x75, 0x08,			// REPORT_SIZE (8bit)
			0x95, 0x08,			// REPORT_COUNT (8 fields)
			0x91, 0x08,			// OUTPUT (Data, Variable, Absolute)
		0xC0,					// END_COLLECTION
	0xC0					// END_COLLECTION (Application)
};

uchar usbFunctionWrite(uchar *data, uchar len)
{
	static uchar page_mode;
	static uint16_t page_addr;
	uchar i;
	uint8_t cmd = data[0]&0xfe;

	if (data[0]&1) {
//		usbSetInterrupt(usbData, 8);
	}

	return 8;
}


usbMsgLen_t usbFunctionSetup(uchar data[8])
{
    usbRequest_t    *rq = (void *)data;
    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* HID class request */
	    if(rq->bRequest == USBRQ_HID_SET_REPORT)
          return USB_NO_MSG;  /* use usbFunctionWrite() to receive data from host */
    }
    return 0;
}



/////////////////////////// LCD Functions /////////////////////////////
#define LCD_PORT PORTD
#define LCD_SIG_RS 0x10
#define LCD_SIG_E  0x20
#define LCD_CMD_CLEAR 0x01

volatile void lcd_wait(void)
{
	_delay_us(40);
}

// 8rbgo
void lcd_out(unsigned char d)
{
	LCD_PORT = d | LCD_SIG_E; // output and enable E
	lcd_wait();
	LCD_PORT &=  ~LCD_SIG_E; // output and disable E
	lcd_wait();
}

// LCD
void lcd_init(void)
{
	// 3 00000011o
	lcd_out(0x03);
	_delay_ms(5);
	lcd_out(0x03);
	_delay_ms(2);
	lcd_out(0x03);
	lcd_wait();

	// t@NVZbg
	lcd_out(0x02); // 4rbg[h
	lcd_out(0x2);
	lcd_out(0x8);
	lcd_wait();

	// 00001100 \I/J[\OFF/uNOFF
	lcd_out(0x0);
	lcd_out(0xc);

	// 00000110 Gg[[hZbg
	lcd_out(0x0);
	lcd_out(0x6);
}

// f[^o
void lcd_write(unsigned char d)
{
	lcd_out(((d >> 4)&0x0f) | LCD_SIG_RS);  // 4rbg
	lcd_out((d&0x0f) | LCD_SIG_RS);       // 4rbg
	lcd_wait();
}

// \NA
void lcd_clear(void)
{
	LCD_PORT = 0;
	lcd_out(0);
	lcd_out(LCD_CMD_CLEAR);
	_delay_ms(2);
}


void lcd_setpos(int x,int y)
{
	y &= 0x1; // 2ŝ
	LCD_PORT = 0;
	lcd_out(0x08 | (y << 2) );
	lcd_out(x);
	_delay_ms(1);
}

static int lcd_line = 0;
static int lcd_clearflag = 0;

// charM
void lcd_putchar(char c) {
	if (c=='\n') {
		lcd_line++;
		lcd_clearflag = 1;
	} else if (c=='\f') {
		lcd_clear();
		lcd_line = 0;
	} else {
		if (lcd_clearflag) {
			lcd_setpos(0,lcd_line);
			lcd_clearflag = 17;
			while (--lcd_clearflag) lcd_write(' ');
			lcd_setpos(0,lcd_line);
		}
		lcd_write((unsigned char)c);
	}
}

// o
void lcd_puts(char *s) {
	while (*s) lcd_putchar(*s++);
}


// lM
void lcd_puti(long d)
{
	char s[8];
	int l=0,sgf=0;
	s[sizeof(s)-1]='\0';
	if (d<0) {
		d=-d;
		sgf=1;
	}
	do{
		s[sizeof(s)-2-l]=d%10+'0';
		l++;
	} while(d/=10);
	if (sgf) {
		s[sizeof(s)-2-l]='-';
		l++;
	}
	lcd_puts(s+(sizeof(s)-1-l));
}
///////////////////////////////////////////////////////////////////////


// w肳ꂽԂ҂
void delay_ms(uint16_t w){
	while (w-->0) _delay_ms(1);
}


// A/D 
void ad_init(void)
{
	ADMUX = (1<<ADLAR); // 
	ADMUX |= (1<<REFS0); //d
	
	ADCSRA |= (1<<ADEN); // A/Dϊ
	ADCSRA |= (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0); //A/DϊNbNI
	
	DIDR0 = (1<<ADC0D) | (1<<ADC1D) | (1<<ADC2D); // PC0,1,2 fW^͋֎~
}

// A/D ϊ
uint8_t ad_conv(int8_t n)
{
	ADMUX = (ADMUX&0xF0) | n; // sI
	ADCSRA |= (1<<ADSC); // ϊJn
	while (ADCSRA & 1<<ADSC) ; // ϊ҂

	return (uint8_t)ADCH;
}



void pwm_init()
{
	TCNT0 = 0x00; //JE^
	TCCR0A  = (1<<COM0A1);
	TCCR0A |= (1<<WGM00); //ʑPWM
	TCCR0B  = (1<<CS01); //Ou1/8
}



int main(void)
{
	DDRB = 0x04;
	PCMSK0=2;

	DDRD = 0xff;  // LCD
	delay_ms(10); // ҂
	lcd_init();
	lcd_clear();

	// obNCgPWMŐ
	pwm_init();
	OCR0A=0x00;

	// Xɖ邭
	uint8_t i;
	for (i=0;i<255;i++) {
		delay_ms(5);
		OCR0A=i;
	}

	delay_ms(50);

	lcd_puts("Stand by ready!\n");

	delay_ms(500);

	lcd_puts("Set up.");


	PORTB |= 0x04; //USB ENABLE

	usbInit();
	sei();

	for(;;){    /* main event loop */
		usbPoll();
    }

}

